<?php
/** @noinspection PhpUnusedPrivateFieldInspection */
declare(strict_types=1);

namespace IntiTest\Test\TestCase;

use Doctrine\DBAL\Exception as DBALException;
use Inti\Test\TestCase\WordPressTestCase;
use WP_User;

use function defined;
use function function_exists;
use function get_option;
use function get_user_by;
use function realpath;
use function update_option;

/**
 * Class WordPressTest
 *
 * @package IntiTest\Test\TestCase
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-test
 */
final class WordPressTest extends WordPressTestCase
{
    const BLOGNAME = 'some random string';

    protected array $config = [
        'db_connections' => [
            'default' => [
                'host' => 'localhost',
                'user' => 'root',
                'password' => 'password',
                'dbname' => 'inti_wp_test',
            ],
        ],
    ];

    protected array $wp_options = [
        'blogname' => self::BLOGNAME,
    ];

    protected function setUp(): void
    {
        parent::setUp();
        $this->runApplication();
    }

    public function testABSPATHIsDetected()
    {
        $expected = realpath(__DIR__ . '/../../wordpress') . '/';

        $this->assertTrue(defined('ABSPATH'));
        /** @noinspection PhpUnitMisorderedAssertEqualsArgumentsInspection */
        $this->assertEquals($expected, ABSPATH);
    }

    public function testCanRunWordPress()
    {
        $this->assertTrue(function_exists('get_user_by'));

        $user = get_user_by('ID', 1);
        $this->assertInstanceOf(WP_User::class, $user);
        $this->assertEquals('admin', $user->user_login);
    }

    /**
     * @throws DBALException
     */
    public function testCanOverrideWpOptions()
    {
        $conn = $this->createConnection();
        $result = $conn->fetchOne('SELECT option_value FROM wp_options WHERE option_name = "blogname"');
        $this->assertNotEquals(self::BLOGNAME, $result);

        $this->assertEquals(self::BLOGNAME, get_option('blogname'));
    }

    /**
     * @throws DBALException
     */
    public function testWillRevertUpdatesAfterTest()
    {
        $conn = $this->createConnection();
        $result = $conn->fetchOne('SELECT option_value FROM wp_options WHERE option_name = "mailserver_url"');
        $this->assertEquals('mail.example.com', $result);

        $isUpdated = update_option('mailserver_url', self::BLOGNAME);
        $this->assertTrue($isUpdated);

        $option_value = get_option('mailserver_url');
        $this->assertEquals(self::BLOGNAME, $option_value);
    }
}
